/*
 *++
 * NAME
 *    rcons 8
 * SUMMARY
 *    Small tool to redirect /dev/console
 * SYNOPSYS
 *    rcons [-r] _[< pty]_
 * DESCRIPTION
 *    Redirects the output of =/dev/console= to the pty opened on STDIN.
 * OPTIONS
 *    * *-r*
 *      Used *-r* to stop redirection.
 * BUGS
 *    This only works on pty's.
 * AUTHORS
 *    Alejandro Liu Ly
 *--
 */
#include <unistd.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>

void pfatal(const char *str,int num) {
  perror(str);
  exit(num);
}

int main(int argc,char **argv) {
  int fd;
  if (!(argc==2 && argv[1][0]=='-' && argv[1][1]=='r' && argv[1][2]=='\0') &&
      !(argc == 1)) {
    fprintf(stderr,"Usage: %s [-r] < {pty}\n",argv[0]);
    exit(3);
  }

  /* Reset Redirection... */
  fd = open("/dev/console",O_RDONLY);
  if (fd == -1) pfatal("open(\"/dev/console\")",1);
  (void)ioctl(fd,TIOCCONS,NULL);

  /* Redirect to stdin... */
  if (argc == 1) {
    if (ioctl(0,TIOCCONS,NULL)) pfatal("ioctl-TIOCCONS",2);
  }
  exit(0);
}
